function [kc0, C0]=clusuplhm(X, kc, nmiss, iter)

%This is the revised version for clusuplh.m so it can handle 
%missing values from phenotype data set.
%
%CLUSUPLM is for updating clustering using likelihood (probability).
%The sample will go to the cluster with the maximum likelihood.
%
% X: mxn data matrix (m observations, n variables). 
% kc: index vector format of raw clusters.
% nmiss: the number of observations (tumors) with missing phenotype data.
% iter: the maximum iteration number (default is 100).
%-------------------------------------------------------------------
%kc0: the updated clusters in index vector format.
%updating clustering using likelihood.

if nargin < 4
    iter=100;
end

initstate(123);
[m,n]=size(X); C=clusindex(kc); kc0=kc;
K=length(C); missk=randsample(1:K, nmiss); 
%initially assign missing observations to clusters.
for j=1:nmiss
    tn=m-nmiss+j;
    kc0(tn, :)=[tn, missk(j)];
end
C0=clusindex(kc0);

%updating clusters using likelihood method.
for i=1:iter
    LK=likelihm(X,C0); 
    LR=[]; kmax=[];
    LKmax=max(LK, [], 2);
    for t=1:m
        tkc=kc0(t,2);
        kmax0=find(LK(t,:)==LKmax(t));
        kmax(t,1)=randsample(kmax0, 1);
        LR(t,1)=LKmax(t)/LK(t,tkc);
    end
    
    %looking for the sample needing to move;
    mLR=max(LR);
    if mLR<=1
        disp(sprintf('Updating coverges at %d iterations !!!!!!!!!',i));
        break
    else
        t0=find(LR==mLR); t1=t0(1);
        k1=kc0(t1,2); k2=kmax(t1); C0k=C0{k1};
        k10=find(C0k==t1); C0k(k10)=[]; C0{k1}=C0k; C0{k2}=[C0{k2},t1];
        kc0(t1,2)=k2;
    end
    [k1 k2 t1]
disp(sprintf('Updating does not coverge at %d iterations',i));
end

%-------------------------------------------------------------------------%
function LK = likelihm(X,C);
%%Calaulate the likelihood for each point in X with cluster C;


[m,n]=size(X); Xn0=min(X,[],1); X=X-Xn0(ones(m,1),:); Xm=max(X,[],1);
K=length(C); LK=ones(m,K);

%Estimate the parameters for each cluster.
p=cell(1);
for k=1:K; mk=length(C{k}); Xk=X(C{k},:);
        for i=1:n
            for j=0:Xm(i)
                y=sum(Xk(:,i)==j); p{k,i}(j+1)=y/mk;
            end
        end
end
     
%calculating likelihood for each point in Xk;
for t=1:m
    j0=X(t,:)+1;
    for k=1:K
        for i=1:n
            j0i=j0(i);
            if ~isnan(j0i)
                LK(t,k)=LK(t,k)*p{k,i}(j0i);
            end
        end
    end
end

%----------------------------------------------------------------------
function initstate(state)

rand('state', state);
randn('state', state);