function [C0,C] = lhupdate(X,kc,iter);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%This program is for clustering samples with pheno and epigenotype data for
% breast tumor progression pathway using "Heritable clustering approach".
%-------------------------------------------------------------------------%
%This code uses likelihood probability to update the clusters............
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if nargin < 3
    iter=100;
end

if nargin < 2
    load indexkm.txt; 
    kc=indexkm;
end

if nargin < 1
    load tumordata2; 
    X=tumordata2(:,2:15);
end

[m,n]=size(X);C=clusindex(kc);
%-------------------------------------------------------------------------%
%updating clustering using likelihood.
C0=C;
for i=1:iter
    i
    kc0=[];
    LK=likelih(X,C0);
    LKmax=max(LK, [], 2);
    for t=1:m
        kmax=find(LK(t,:)==LKmax(t));
        kc0(t,1)=t; kc0(t,2)=kmax(1);
    end
    if kc0==kc
        break
    else
        C0=clusindex(kc0); kc=kc0;
    end
end

%-------------------------------------------------------------------------%
function indc = indexclus(C)
%transfer cluster C to index;

lenC=length(C); indc=[];
for k=1:lenC
    X=C{k}; lenX=length(X);
    for j=1:lenX
        indc = [indc; [X(j), k]];
    end
end
indc = sortrows(indc, 1);  

%-------------------------------------------------------------------------%
function C=clusindex(KC)
%This function is used for transfering index in KC to cluster C as cell;
C=cell(1);
K=max(KC(:,2));
for k=1:K
    ind=find(KC(:,2)==k); C{k}=ind';
end

%-------------------------------------------------------------------------%
function LK = likelih(X,C);
%%Calaulate the likelihood for each point in X with cluster C;
%%The minimum value of X in each dimension (variable) is 0.

[m,n]=size(X); Xn0=min(X,[],1); Xnm=max(X,[],1);
K=length(C); T=1:m; LK=ones(m,K);

%Estimate the parameters for each cluster.
p=cell(1);
for k=1:K; mk=length(C{k}); Xk=X(C{k},:);
        for i=1:n
            for j=Xn0(i):Xnm(i)
                y=sum(Xk(:,i)==j); p{k,i}(j+1)=y/mk;
            end
        end
end
     
%calculating likelihood for each point in Xk;
for t=1:m
    j0=X(t,:)+1;
    for k=1:K
        for i=1:n
            j0i=j0(i);
            LK(t,k)=LK(t,k)*p{k,i}(j0i);
        end
    end
end
