function [currentn, childnode, NC, Pheno] = pathway(X,p,kC,hepsi)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%This program is for recapitulating the progression pathways of    %
% breast tumor using any clustering results in kC for data set X   % 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


if nargin < 2
    X=load('BreastWork55.txt'); p=5;
end
if nargin < 3
    kC=load('idxcsi');
end
if nargin < 4
    hepsi = 1;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[m,n]=size(X); p1=p+1; T=1:m;
X1=X(:, 1:p); %Phenotype data set;
X2=X(:, p1:n); %Genotype data set;
C=clusindex(kC); lenC=length(C);

%-------------------------------------------------------------------------%
%%Calaulation of genocenter;
NC=[]; 
for k=1:lenC
    t=C{k}; lenCk=length(t); 
    if lenCk==1
        NC(k, :)=X2(t, :);
    else
        X0=sum(X2(t, :)); lenCk2=lenCk/2;
        for j=1:length(X0)
            if X0(j)>=lenCk2; NC(k,j)=1;
            else NC(k,j)=0; 
            end
        end
    end
end
%-------------------------------------------------------------------------%

%%Calaulation of phenocenters and scores;
PC=[];
for k=1:lenC; lenCk=length(C{k}); t=C{k};
    px=X1(t, :); PC(k, :)=round(mean(px,1));
end
maxpc=max(X1,[],1); minpc=min(X1,[],1); dpc=maxpc-minpc; SPC=[];
for k=1:lenC
    SPC(k,:)=(PC(k,:)-minpc)./dpc;
end
PhenoS=mean(SPC, 2);    
%-------------------------------------------------------------------------%        
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%PathwayDiscovery---Building tumor progression tree.%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Find the root node:
lenC=length(C); nc1=sum(NC,2); 
nc=[(1:lenC)', nc1, PhenoS]; 
nc0=sortrows(nc, [3,2]);
rootnode=nc0(1,1); nc0(1,:)=[]; 

%find new node (child node).
currentn=cell(1); childnode=[]; currentn(1)={rootnode}; 
while lenC>1
    lenC=lenC-1; newnode=nc0(1,1); kc=length(currentn); nc0(1,:)=[]; 
    while kc>0; kcc=length(currentn{kc}); nchild=[];
        for kk=1:kcc
           current1=currentn{kc}(kk); 
           ac=find(NC(current1,:)==1); an=find(NC(newnode,:)==1);
           ac1=sum(ismember(ac, an)); lac=length(ac);
           bc=PC(current1,:); bn=PC(newnode,:); bcn=sum(bc<=bn);
           if ac1>=hepsi*lac & bcn==p
               nchild=[nchild, kk]; 
           end
        end
        
        lnchild=length(nchild);
        if isempty(nchild)
            kc=kc-1;
        else
            if lnchild > 1; 
                cchild=currentn{kc}(nchild);
                cc1=[nc1(cchild), PhenoS(cchild), nchild']; 
                cc2=sortrows(cc1, [1, 2]); 
                nchild=cc2(lnchild, 3);
            end
             childnode=[childnode; [kc, nchild, newnode]];
            if length(currentn)<kc+1
                currentn(kc+1)={newnode};
            else currentn(kc+1)={[currentn{kc+1}, newnode]};
            end 
            break 
        end
    end
    if kc==0; currentn(1)={[currentn{1}, newnode]}; 
        rootnode=0;
    end;
end
Pheno=[PC, PhenoS];
