function result = wepsilon(X,p)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%This program is for clustering samples with pheno and epigenotype data for
% breast tumor progression pathway using "Heritable clustering approach".   
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if nargin < 2
    X=load('BreastWork55.txt'); p=5;
end
result=[];
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[m,n]=size(X); p1=p+1; T=1:m;
X1=X(:, 1:p); %Phenotype data set;
X2=X(:, p1:n); %Genotype data set;

X1min=min(X1, [], 1); X1=X1-X1min(ones(m,1),:);
X1max=max(X1, [], 1); X1=X1./X1max(ones(m,1),:);

%calculating distance;
    tt1=[]; tt2=[]; dp=[]; dg=[]; 
for i=1:(m-1)
    for j=(i+1):m
        tt1=[tt1; T(i)]; tt2=[tt2; T(j)];
        dp=[dp; sum(abs(X1(i,:)-X1(j,:)))/p];
        dg=[dg; sum(abs(X2(i,:)-X2(j,:)))/(n-p)];
    end
end

for w=0.2:0.1:0.8
        d=w*dp+(1-w)*dg;
        D=[tt1,tt2,1-d];

D1=sortrows(D, 3); 

for epsi=0.5:0.1:1

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%---------------------Begin Clustering--------------------%
T=1:m; C=cell(1); %keep the clusters with their tumors within.
if D1(1,3)>=epsi
    input('Epsilon is too small, all tumors are in one cluster.');
else
    T1=D1(1,1); T2=D1(1,2); C(1)={T1}; C(2)={T2};
    t0=find(T==T1 | T==T2); T(t0)=[]; lenT=length(T);
    while lenT>0
        lenC=length(C); 
        Ts=[];%keep tumor for adding to cluster.
        for k=1:lenC; s0=[]; 
            lenCk=length(C{k}); 
            for j=1:lenCk; 
                for t=1:lenT
                t1=find((D(:,1)==T(t) & D(:,2)==C{k}(j)) | ...
                    (D(:,1)==C{k}(j) & D(:,2)==T(t)));
                s0(t,j)=D(t1,3);
                end
            end
            %find the tumor could be clustered to Cluster K.
            T0=T; s01=min(s0,[],2); s02=mean(s0, 2);
            t01=find(s01>=epsi); 
            if isempty(t01) 
                Ts(k, :)=[0 0];
            else s2=s02(t01); T0=T0(t01);
                t02=find(s2==max(s2));
                t02=t02(1);
                Ts(k,:)=[T0(t02), s2(t02)];
            end
        end
        maxs=max(Ts(:,2));
        if maxs>0
            t5=find(Ts(:,2)==maxs); 
            t5=t5(1);
            T01=Ts(t5,1);
            C(t5)={[C{t5}, T01]}; 
            t0=find(T==T01); T(t0)=[];
        else 
            C(lenC+1)={T(1)}; T(1)=[];
        end
        lenT=length(T);
    end
end %ending clustering
%-------------------------------------------------------------------------%
AS=asimilar(C,D); TS=sum(AS); lenC=length(C);
f=log(TS)-lenC/n;
result=[result; [w epsi lenC TS f]];
[w, epsi, lenC, TS, f]
end
end
result=sortrows(result, 5);

%-------------------------------------------------------------------------%
function AS = asimilar(C,D);
%%Calaulate average similarity within each cluster (AS);
AS=[]; lenC=length(C);
for k=1:lenC; lenCk=length(C{k});
    t=sort(C{k});
    if lenCk==1
        AS(k)=1;
    else
        AS(k)=lenCk;
        for i1=1:lenCk-1
            for i2=(i1+1):lenCk
                i0=find(D(:,1)==t(i1) & D(:,2)==t(i2));
                AS(k)=AS(k)+2*D(i0,3);
            end
        end
        AS(k)=AS(k)/lenCk^2;
    end
end
